#########################################################################
# test_task_appendantab.py
# Copyright (C) 2018
# Associated Universities, Inc. Washington DC, USA.
#
# This script is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
# License for more details.
#
#
# Based on the requirements listed in casadocs found here:
# 
#
##########################################################################
import os
import shutil
import unittest
from unittest.mock import patch

import numpy as np

from casatestutils import testhelper as th

from casatasks import appendantab
from casatools import ctsys, table

tb = table()

datapath = ctsys.resolve('unittest/appendantab')

# input data
evn_data = "idi1.ms"
evn_antab = os.path.join(datapath, "n14c3.antab")
append_data = "appendedData.ms"
evn_ref = os.path.join(datapath, "appendantab_test.ms")
data_copy = "dataCopy.ms"
    

def compareGains(table, ref):
    # Compare the gain values
        gains = []
        tb.open(table+"/GAIN_CURVE")
        for i in range(tb.nrows()):
            gains.extend(tb.getcell('GAIN', i))
        tb.close()

        ref_gains = []
        tb.open(ref+"/GAIN_CURVE")
        for i in range(tb.nrows()):
            cur = tb.getcell('GAIN', i)
            ref_gains.extend(tb.getcell('GAIN', i))
        tb.close()

        if len(ref_gains) != len(gains):
            return False

        for i in range(len(gains)):
            #print("COMP: ", ref_gains[i]- gains[i])
            #print("TRUTH: ", np.isclose(ref_gains[i], gains[i], atol=1e-6))
            if not np.all(np.isclose(ref_gains[i], gains[i])):
                print("REF AND REAL DIFFER: ", ref_gains[i], gains[i])
                return False

        return True

def compareTsys(table, ref):
    # Compare the tsys values
        tb.open(table+"/SYSCAL")
        sys = tb.getcol("TSYS")
        tb.close()

        tb.open(ref+"/SYSCAL")
        ref_sys = tb.getcol("TSYS")
        tb.close()

        if len(ref_sys) != len(sys):
            return False

        for i in range(len(sys)):
            if not np.all(np.isclose(ref_sys[i], sys[i])):
                return False

        return True

class appendantab_test(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        pass

    def setUp(self):
        if os.path.exists(data_copy):
            shutil.rmtree(data_copy)
        shutil.copytree(os.path.join(datapath, evn_data), data_copy)

    def tearDown(self):
        if os.path.exists(data_copy):
            shutil.rmtree(data_copy)

        if os.path.exists(append_data):
            shutil.rmtree(append_data)
        
        if os.path.exists("secondAppend.ms"):
            shutil.rmtree("secondAppend.ms")

    @classmethod
    def tearDownClass(cls):
        if os.path.exists(data_copy):
            shutil.rmtree(data_copy)

        if os.path.exists(append_data):
            shutil.rmtree(append_data)

    def test_appendAll(self):
        """ Test appending both the GAIN_CURVE and SYSCAL tables """
        appendantab(vis=data_copy, outvis=append_data, antab=evn_antab, overwrite=False, append_tsys=True, append_gc=True)

        # compare to ref syscal and gain_curve table
        self.assertTrue(compareTsys(append_data, evn_ref))
        self.assertTrue(compareGains(append_data, evn_ref))

    def test_appendGainCurve(self):
        """ Test appending just the GAIN_CURVE table """
        appendantab(vis=data_copy, outvis=append_data, antab=evn_antab, overwrite=False, append_tsys=False, append_gc=True)

        # make sure SYSCAL wasn't created
        self.assertFalse(os.path.exists(append_data + "/SYSCAL"))
        self.assertTrue(compareGains(append_data, evn_ref))

    def test_appendSysCal(self):
        """ Test appending just the SYSCAL table """
        appendantab(vis=data_copy, outvis=append_data, antab=evn_antab, overwrite=False, append_tsys=True, append_gc=False)

        # make sure GAIN_CURVE wasn't created
        self.assertFalse(os.path.exists(append_data + "/GAIN_CURVE"))

        # compare to ref gain_curve table
        self.assertTrue(compareTsys(append_data, evn_ref))

    #@unittest.skip("Will develop more on tarball")
    def test_appendToExistingSyscal(self):
        """ Test that the task appends data to an existing SYSCAL table """
        appendantab(vis=data_copy, outvis=append_data, antab=evn_antab, overwrite=False, append_tsys=True, append_gc=False)
        # Get the number of rows added
        tb.open(append_data + "/SYSCAL")
        first_run = tb.nrows()
        tb.close()

        appendantab(vis=append_data, outvis="secondAppend.ms", antab=evn_antab, overwrite=False, append_tsys=True, append_gc=False)
        # Get the number of rows after consecutive runs
        tb.open("secondAppend.ms/SYSCAL")
        next_run = tb.nrows()
        tb.close()

        self.assertTrue(next_run > first_run)

    #@unittest.skip("skip for now")
    def test_appendToExistingGainCurve(self):
        """ Test that the task appends data to an existing GAIN_CURVE table """
        appendantab(vis=data_copy, outvis=append_data, antab=evn_antab, overwrite=False, append_tsys=False, append_gc=True)
        # Get the number of rows added
        tb.open(append_data + "/GAIN_CURVE")
        first_run = tb.nrows()
        tb.close()

        appendantab(vis=append_data, outvis="secondAppend.ms", antab=evn_antab, overwrite=False, append_tsys=False, append_gc=True)
        # Get the number of rows after consecutive runs
        tb.open("secondAppend.ms/GAIN_CURVE")
        next_run = tb.nrows()
        tb.close()

        #print("FIRST: ", first_run, " AFTER: ", next_run)
        self.assertTrue(next_run > first_run)



    def test_Overwrite(self):

        # Don't overwrite the data on the first run
        appendantab(vis=data_copy, outvis=append_data, antab=evn_antab, overwrite=False, append_tsys=True, append_gc=False)

        # Remove subtables so they are re-written in the overwrite
        shutil.rmtree(append_data + "/SYSCAL")

        # Second run with overwrite should create subtables
        appendantab(vis=data_copy, outvis=append_data, antab=evn_antab, overwrite=True, append_tsys=True, append_gc=False)

        # Subtables should be created
        self.assertTrue(os.path.exists(append_data + "/SYSCAL"))





if __name__ == '__main__':
    unittest.main()
